@@ -17,12 +17,14 @@
 import doctest
 import os.path
 import subprocess
+import sys
 
 from testtools import (
     TestCase,
     )
 from testtools.matchers import (
     DocTestMatches,
+    MatchesAny,
     )
 
 class TestCanSetup(TestCase):
@@ -30,10 +32,20 @@ class TestCanSetup(TestCase):
     def test_bdist(self):
         # Single smoke test to make sure we can build a package.
         path = os.path.join(os.path.dirname(__file__), '..', '..', 'setup.py')
-        proc = subprocess.Popen([path, 'bdist'], stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        proc = subprocess.Popen([sys.executable, path, 'bdist'],
+            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT, universal_newlines=True)
         output, _ = proc.communicate()
         self.assertEqual(0, proc.returncode)
-        self.assertThat(output, DocTestMatches("""...
+        self.assertThat(output, MatchesAny(
+            # win32
+            DocTestMatches("""...
+running install_scripts
+...
+adding '...testr'
+...""", doctest.ELLIPSIS),
+            # unixen
+            DocTestMatches("""...
 ...bin/testr ...
-""", doctest.ELLIPSIS))
+""", doctest.ELLIPSIS)
+            ))
