@@ -19,10 +19,12 @@ import optparse
 import subprocess
 import sys
 
+from testtools.matchers import raises
+
 from testrepository import arguments, commands
 import testrepository.arguments.command
 from testrepository.commands import load
-from testrepository.ui import cli, model
+from testrepository.ui import cli, decorator, model
 from testrepository.tests import ResourcedTestCase
 
 
@@ -46,10 +48,16 @@ def cli_ui_factory(input_streams=None, options=(), args=()):
     return cli.UI(argv, stdin, stdout, stderr)
 
 
+def decorator_ui_factory(input_streams=None, options=(), args=()):
+    base = model.UI(input_streams=input_streams, options=options, args=args)
+    return decorator.UI(input_streams=input_streams, decorated=base)
+
+
 # what ui implementations do we need to test?
 ui_implementations = [
     ('CLIUI', {'ui_factory': cli_ui_factory}),
     ('ModelUI', {'ui_factory': model.UI}),
+    ('DecoratorUI', {'ui_factory': decorator_ui_factory}),
     ]
 
 
@@ -87,7 +95,7 @@ class TestUIContract(ResourcedTestCase):
 
     def test_iter_streams_unexpected_type_raises(self):
         ui = self.get_test_ui()
-        self.assertRaises(KeyError, ui.iter_streams, 'subunit')
+        self.assertThat(lambda: ui.iter_streams('subunit'), raises(KeyError))
 
     def test_output_error(self):
         try:
@@ -102,14 +110,6 @@ class TestUIContract(ResourcedTestCase):
         ui = self.get_test_ui()
         ui.output_rest('')
 
-    def test_output_results(self):
-        # output_results can be called and takes a thing that can be 'run'.
-        ui = self.get_test_ui()
-        class Case(ResourcedTestCase):
-            def method(self):
-                pass
-        ui.output_results(Case('method'))
-
     def test_output_stream(self):
         # a stream of bytes can be output.
         ui = self.get_test_ui()
@@ -188,7 +188,32 @@ class TestUIContract(ResourcedTestCase):
     def test_exec_subprocess(self):
         # exec_subprocess should 'work like popen'.
         ui = self.ui_factory()
-        proc = ui.subprocess_Popen('ls', stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
+        proc = ui.subprocess_Popen([sys.executable, "-V"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out, err = proc.communicate()
         proc.returncode
+
+    def test_subprocesses_have_stdin(self):
+        # exec_subprocess should 'work like popen'.
+        ui = self.ui_factory()
+        proc = ui.subprocess_Popen([sys.executable, "-V"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        proc.stdout.read(0)
+        out, err = proc.communicate()
+
+    def test_subprocesses_have_stdout(self):
+        # exec_subprocess should 'work like popen'.
+        ui = self.ui_factory()
+        proc = ui.subprocess_Popen([sys.executable, "-V"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        proc.stdout.read(0)
+        out, err = proc.communicate()
+
+    def test_make_result(self):
+        # make_result should return a TestResult.
+        ui = self.ui_factory()
+        ui.set_command(commands.Command(ui))
+        result = ui.make_result(lambda: None)
+        result.startTestRun()
+        result.stopTestRun()
+        self.assertEqual(0, result.testsRun)
